/***** BEGIN LICENSE BLOCK *****
   - Version: MPL 1.1/GPL 2.0/LGPL 2.1
   -
   - The contents of this file are subject to the Mozilla Public License Version
   - 1.1 (the "License"); you may not use this file except in compliance with
   - the License. You may obtain a copy of the License at
   - http://www.mozilla.org/MPL/
   -
   - Software distributed under the License is distributed on an "AS IS" basis,
   - WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
   - for the specific language governing rights and limitations under the
   - License.
   -
   - The Original Code is _by Sudhakar "Thaths" Chandrasekharan_.
   -
   - The Initial Developer of the Original Code is
   - _Sudhakar "Thaths" Chandrasekharan <thaths@openscroll.org>_.
   - Portions created by the Initial Developer are Copyright (C) 2003-2004
   - the Initial Developer. All Rights Reserved.
   -
   - Contributor(s):
   - Sudhakar "Thaths" Chandra <thaths@openscroll.org> (original author)
   -
   - Alternatively, the contents of this file may be used under the terms of
   - either the GNU General Public License Version 2 or later (the "GPL"), or
   - the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
   - in which case the provisions of the GPL or the LGPL are applicable instead
   - of those above. If you wish to allow use of your version of this file only
   - under the terms of either the GPL or the LGPL, and not to allow others to
   - use your version of this file under the terms of the MPL, indicate your
   - decision by deleting the provisions above and replace them with the notice
   - and other provisions required by the LGPL or the GPL. If you do not delete
   - the provisions above, a recipient may use your version of this file under
   - the terms of any one of the MPL, the GPL or the LGPL.
   -
   - ***** END LICENSE BLOCK *****/

/**************************************************************************
* The following is to enable dumps from inside JS.
**************************************************************************/
var debugboolean = false;
//setting pref for debug
if(debugboolean==true)
{
    const PREFS_CID      = "@mozilla.org/preferences;1";
    const PREFS_I_PREF   = "nsIPref";
    const PREF_STRING    = "browser.dom.window.dump.enabled";
    try {
	var Pref        = new Components.Constructor(PREFS_CID, PREFS_I_PREF);
	var pref        = new Pref( );
	pref.SetBoolPref(PREF_STRING, true);
    } catch(e) { 
	alert(e);
    }
}

// Some XPCOM constants we need to walk the bookmarks tree
// Commented out because they are already defined in chrome://communicator/content/bookmarks/bookmarks.js
// const RDF = Components.classes['@mozilla.org/rdf/rdf-service;1'].getService(Components.interfaces.nsIRDFService);
// const RDFC = Components.classes['@mozilla.org/rdf/container;1'].createInstance(Components.interfaces.nsIRDFContainer);
// const RDFCU = Components.classes['@mozilla.org/rdf/container-utils;1'].getService(Components.interfaces.nsIRDFContainerUtils);
// const XUL_NS = "http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul";
// const NC_NS = "http://home.netscape.com/NC-rdf#";
// const BMDS = Components .classes["@mozilla.org/rdf/datasource;1?name=bookmarks"].getService(Components.interfaces.nsIRDFDataSource); 
// const RDF_NS = "http://www.w3.org/1999/02/22-rdf-syntax-ns#";

// Some global variables that are used across functions
var feedsTree;		// The tree used to display the feeds
var xmlhttp;		// A HTTP request object
var localPath;		// Path to feeds downloaded into a local file
var win;		// The Habari Xenu window
var progmtr;		// The progress meter
var updateInterval;	// How often to update
var showDetail;		// Should we show details about the feed
var fetchURL;		// The Feed's URL
var initialized = false;    // amiinitializedornot.com
var prefFolder;		// The bookmark folder that contains bookmarks to all the RSS feeds
var statusLabel;
var lastUpdatedAt;
var strbundle;
var selectURN;
var freshDownload;
var favIcon;

// HTTP request results
const RESULT_OK = 0;
const RESULT_PARSE_ERROR = 1;
const RESULT_NOT_RSS = 2;
const RESULT_NOT_FOUND = 3;
const RESULT_NOT_AVAILABLE = 4;
const RESULT_ERROR_FAILURE = 5;

/**************************************************************************
* Populate the tree from the Datasource. See above for how we get 'ds'
* This function gets called on the XUL window's onload event
**************************************************************************/
function buildTreeFromDataSource() {
    dump("buildTreeFromDataSource(): Beginning to build subscription tree from bookmarks\n");

    // strbundle=document.getElementById("_stringbundle");
    strbundle=srGetStrBundle("chrome://habarixenu/locale/strings.properties");

    // Set the feedsTree to point to the tree in which feeds are stored
    feedsTree = document.getElementById("feedsTree");

    /*******************************************************************************
    * We have moved the following bits here because this seems to be the only way
    * the main screen can update itself when changes are made in the settings editor
    *******************************************************************************/
    // Has user selected one folder as preferred feeds folder
    prefFolder = getPrefValue("extensions.habarixenu.feeds_folder", "str");
    updateInterval = getPrefValue("extensions.habarixenu.update_interval", "int", 60);
    showDetail = getPrefValue("extensions.habarixenu.show_details", "bool", false);

    // If we don't run the following only once, it goes into a loop
    // for some reason, buildTreeFromDataSource() gets called a few times
    // when the application is loaded.  Probably because the onload event
    // on the feedsTree happens whenever the browser finishes re-painting.
    // TODO Figure out this is seperated out of the following if block
    if (! initialized) {
	dump("buildTreeFromDataSource(): Not initialized.  Initializing...\n");

	// Initialize win to point to the main window
	win = document.getElementsByTagName("window")[0];
	progmtr = document.getElementById("progmtr");

	// Set the progress meter
	busy();

	// Initialize some stuff in chrome://communicator/content/bookmarks/bookmarks.js
	initServices();
	initBMService();

	// Set the ref attribute on the tree and rebuild it
	feedsTree.setAttribute("ref", prefFolder);
	feedsTree.builder.rebuild();
	statusLabel = document.getElementById("statusLabel");

	// Unset the progress meter
	free();
    }

    if (initialized) {
	dump("buildTreeFromDataSource(): Already initialized.  Returning false.\n");
	return false;
    } else {
	// First see if there is a XXXXXX.slt/chrome/feeds directory exists
	var du = new DirUtils();
	var userChrome = du.getUserChromeDir();

	var dir = new Dir(userChrome);
	dir.append("habarixenucache");

	if (! dir.exists()) {
	    dump("buildTreeFromDataSource(): Directory habarixenucache does not exist. Creating...\n");
	    try {
		dir.create();
	    } catch (e) {
		dump("buildTreeFromDataSource(): Error creating habarixenucache directory\n");
	    }
	}

	var browser =  document.getElementById("parsed-rss-feed");
	var tb =  document.getElementById("hx-tab-area");

	browser.loadURI("chrome://habarixenu/locale/about.html");

	// prefFolder is missing. This means that HX has not been used before.
	// We will create a Habari Xenu Feeds folder and populate it with a few
	// feeds
	if (! prefFolder) {
	    dump("buildTreeFromDataSource(): No prefFolder set. First time launch? Populating with canned feeds.\n");

	    // Create the folder
	    var newFolder = BMSVC.createFolderInContainer("Habari Xenu Feeds", RDF.GetResource("NC:BookmarksRoot"), null);

	    // Set it in preferences
	    setPrefValue("extensions.habarixenu.feeds_folder", "str", newFolder.Value);

	    /*
	    wizMode = WIZ_MODE_IMPORT;
	    txtImportFile = '';
	    if (navigator.userAgent.indexOf('irefox') == -1) {
		// Mozilla
		var du = new DirUtils();
		var userChrome = du.getUserChromeDir();
		var opmlFile = new File(userChrome);
		opmlFile.append("content");
		opmlFile.append("samplesources.opml");
		alert(opmlFile.path);
		alert("Importing " + opmlFile.path);
		txtImportFile = opmlFile.path;
		finish();
	    } else {
		// Firefox
		var du = new DirUtils();
		var userChrome = du.getMozUserHomeDir();
		var opmlFile = new File(userChrome);
		opmlFile.append("{e4ed8486-d9f2-4624-b977-b1774decad7e}");
		opmlFile.append("content");
		opmlFile.append("samplesources.opml");
		alert(opmlFile.path);
		alert("Importing " + opmlFile.path);
		txtImportFile = opmlFile.path;
		finish();
	    }
	    */

	    // Create a sub-folder for news, blogs
	    var newsFolder = BMSVC.createFolderInContainer("News", newFolder, null);
	    var blogsFolder = BMSVC.createFolderInContainer("Blogs", newFolder, null);

	    // Populate it
	    if (BMSVC.createBookmarkInContainer.length == 7) {
		// Firefix <= 0.8
		BMSVC.createBookmarkInContainer("BBC World News", "http://news.bbc.co.uk/rss/newsonline_uk_edition/world/rss091.xml", null, "hxnew -- unknown", null, newsFolder, null);
		BMSVC.createBookmarkInContainer("Reuters", "http://www.microsite.reuters.com/rss/topNews", null, "hxnew -- unknown", null, newsFolder, null);
		BMSVC.createBookmarkInContainer("Washington Post", "http://www.washingtonpost.com/wp-srv/topnews/rssheadlines.xml", null, "hxnew -- unknown", null, newsFolder, null);

		BMSVC.createBookmarkInContainer("Boing Boing", "http://boingboing.net/rss.xml", null, "hxnew -- unknown", null, blogsFolder, null);
		BMSVC.createBookmarkInContainer("Fark", "http://www.pluck.com/rss/fark.rss", null, "hxnew -- unknown", null, blogsFolder, null);

		BMSVC.createBookmarkInContainer("Habari Xenu News", "http://www.livejournal.com/users/habarixenu/data/atom", null, "hxnew -- unknown", null, newFolder, null);
	    } else {
		// Firefox > 0.8
		BMSVC.createBookmarkInContainer("BBC World News", "http://news.bbc.co.uk/rss/newsonline_uk_edition/world/rss091.xml", null, "hxnew -- unknown", null, null, newsFolder, null);
		BMSVC.createBookmarkInContainer("Reuters", "http://www.microsite.reuters.com/rss/topNews", null, "hxnew -- unknown", null, null, newsFolder, null);
		BMSVC.createBookmarkInContainer("Washington Post", "http://www.washingtonpost.com/wp-srv/topnews/rssheadlines.xml", null, "hxnew -- unknown", null, null, newsFolder, null);

		BMSVC.createBookmarkInContainer("Boing Boing", "http://boingboing.net/rss.xml", null, "hxnew -- unknown", null, null, blogsFolder, null);
		BMSVC.createBookmarkInContainer("Fark", "http://www.pluck.com/rss/fark.rss", null, "hxnew -- unknown", null, null, blogsFolder, null);

		BMSVC.createBookmarkInContainer("Habari Xenu News", "http://www.livejournal.com/users/habarixenu/data/atom", null, "hxnew -- unknown", null, null, newFolder, null);
	    }

	    // Set the ref attribute on the tree and rebuild it
	    feedsTree.setAttribute("ref", newFolder.Value);
	    feedsTree.builder.rebuild();

	    if (confirm(strbundle.GetStringFromName("firsttimeconfirm"))) {
		dump("buildTreeFromDataSource(): First launch of HX. Launching Settings Editor\n");
		openSettingsEditor();
	    }
	}
	initialized = true;

	dump("buildTreeFromDataSource(): Finished initialization.  Returning true.\n");
	return true;
    }
    dump("buildTreeFromDataSource(): Finished rebuilding subscription tree from bookmarks.\n");
}


/**************************************************************************
* This function gets called when the user selects an item in the feeds tree
* Here we find the URL and last update time of the feed.
**************************************************************************/
function fetchFeed(tEvent) {
    dump("fetchFeed(): Entered.\n");

    // A flag to indicate whether we downloaded a fresh local copy of a remote feed
    freshDownload = false;

    // feedsTree = document.getElementById("feedsTree");
    feedsTree = tEvent.target;

    // If nothing was selected we have nothing to do
    if (feedsTree.currentIndex < 0) {
	dump("fetchFeed(): Nothing selected.  Returning.\n");
	dump(feedsTree.currentIndex + "\n");
	return false;
    }

    // If folder was selected we have nothing to do
    fetchURL = feedsTree.view.getCellText(feedsTree.currentIndex, "feedlist-url");
    if (! fetchURL) {
	dump("fetchFeed(): Folder selected.  Returning.\n");
	return false;
    }

    // Activate the progress meter to indicate work happening
    busy();

    // The URN of the bookmark item that has been selected
    selectURN = feedsTree.builderView.getResourceAtIndex(feedsTree.currentIndex).Value;

    hxUpdateFeed.onCheck = function(aName, aURL) {
	statusLabel.value = "Checking" + ": " + aName;
    }

    hxUpdateFeed.onChecked = function(aName, aURL) {
	free();
	hxUpdateFeed.downloadList = new Array();
	hxUpdateFeed.nameList = new Array();
	hxUpdateFeed.urlList = new Array();
	hxUpdateFeed.localPathList = new Array();
	hxUpdateFeed.localHTMLPathList = new Array();
	hxUpdateFeed.descriptionList = new Array();
	hxUpdateFeed.lastModifiedList = new Array();
	statusLabel.value = " ";
    }
    hxUpdateFeed.begin(selectURN, true);

    dump("fetchFeed(): Finished. Exiting.\n");
    return false;   // Returning false because we want this event to bubble up
}


/**************************************************************************
* A utility function that takes a URl, converts into an md5 hash, appends
* it to a local feeds directory location and returns the resulting string
**************************************************************************/
function absPathToLocal(url) {
    dump("absPathToLocal(): Called with " + url + "\n");
    var fu = new FileUtils();
    var hash = hex_md5(url);

    var du = new DirUtils();
    var userChrome = du.getUserChromeDir();
    var cacheFile = new File(userChrome);
    cacheFile.append("habarixenucache");
    cacheFile.append(hash);
    if (! cacheFile.exists()) {
	dump("absPathToLocal(): " + cacheFile.path + " does not exist.  Creating\n");
	cacheFile.create();
    }
    dump("absPathToLocal(): Returning " + cacheFile.path + "\n");
    return cacheFile.path;
}


/**************************************************************************
* A utility function that takes a URL, and opens it in a the current
* iframe browser
**************************************************************************/
function openNewsInTab(locHTMLPath) {
    dump("openNewsInTab(): called with " + locHTMLPath + "\n");

    var tbrowser = document.getElementById("parsed-rss-feed");
    var currentBrowser = tbrowser.browsers[0];
    // currentBrowser.addEventListener("DOMLinkAdded", function(event) { event.preventBubble(); }, false);
    currentBrowser.loadURI(locHTMLPath);
    free();
    return;
}


/*************************************************************************
* A utility function to set the cursor to busy state
*************************************************************************/
function busy() {
    // Activate the progress meter to indicate work happening
    // win.setAttribute("wait-cursor", "true");
    dump("busy: Setting progressmeter to busy\n");
    progmtr = document.getElementById("progmtr");
    progmtr.setAttribute("mode", "undetermined");
}


/*************************************************************************
* A utility function to set the cursor to normal state
*************************************************************************/
function free() {
    // Deactivate the progress meter to indicate work done
    dump("free: Setting progressmeter to normal\n");
    progmtr.setAttribute("mode", "determined");
    progmtr.setAttribute("value", "0%");
    // win.removeAttribute("wait-cursor");
}


/**************************************************************************
* This function gets called when we need to open the settings editor
**************************************************************************/
function openSettingsEditor() {
    dump("openSettingsditor(): Opening Settings editor\n");

    // Open a new modal dialog showing the settingseditor xul as chrome
    // pass it a reference to the feedsTree
    var settingsURL = "chrome://habarixenu/content/settingseditor.xul";
    window.openDialog(settingsURL,"","chrome,modal, width=450, height=450", feedsTree);

    dump("openSettingsditor(): Finished opening Settings editor\n");
    return true;
}


/**************************************************************************
* This function gets called when we need to open the bookmarks editor
**************************************************************************/
function openBookmarksEditor() {
    dump("openBookmarksEditor(): Opening Bookmarks editor\n");

    // Open a new window showing the bookmark editor xul as chrome
    var settingsURL;
    if (navigator.userAgent.indexOf("irefox") == -1) {
	// Mozilla
	settingsURL = "chrome://communicator/content/bookmarks/bookmarksManager.xul";
    } else {
	// Firefox
	settingsURL = "chrome://browser/content/bookmarks/bookmarksManager.xul";
    }
    window.openDialog(settingsURL,"","chrome,all,dialog=no,modal, width=340, height=270", prefFolder);

    dump("openBookmarksEditor(): Finished opening Bookmarks editor\n");
    return true;
}


/**************************************************************************
* Close the application
**************************************************************************/
function closeHabariXenu() {
    dump("closeHabariXenu(): Closing window\n");
    var b = parent.getBrowser();
    b.removeCurrentTab();
}


/**************************************************************************
* This function Unsets the LastVisitDate for the selected Feed
**************************************************************************/
function forgetLastVisitDate() {
    feedsTree = document.getElementById("feedsTree");
    // feedsTree = tEvent.target;

    // If nothing was selected we have nothing to do
    if (feedsTree.currentIndex < 0) { return; }

    // If folder was selected we have nothing to do
    fetchURL = feedsTree.view.getCellText(feedsTree.currentIndex, "feedlist-url");
    if (! fetchURL) { return; }

    // The URN of the bookmark item that has been selected
    selectURN = feedsTree.builderView.getResourceAtIndex(feedsTree.currentIndex).Value;

    var rSource = RDF.GetResource(selectURN);
    var rName = RDF.GetResource(WEB_NS + "LastVisitDate");
    var node =  BMDS.GetTarget(rSource, rName, true);

    dump("Calling setBMDSProperty\n");
    setBMDSProperty(rSource, rName);
    dump("Got back from setBMDSProperty\n");

    /*
    if (node) {
	// A LastVisitDate exists for this bookmark.  Change it
	try {
	    BMDS.Unassert(rSource, rName, node);
	    BMDS.QueryInterface(Components.interfaces.nsIRDFRemoteDataSource).Flush();
	} catch (e) {
	    dump("forgetLastVisitDate(): Error resetting LastVisitDate of bookmark: " + e + "\n");
	}
    }
    */
    return;
}


/**************************************************************************
* This functio gets called when the user clicks on the 'Update Feeds'
* button. It initializes a hxUpdateFeed object and calls its begin() method
**************************************************************************/
function updateCheck() {
    busy();
    hxUpdateFeed.onCheck = function(aName, aURL) {
	statusLabel.value = "Checking" + ": " + aName;
    }

    hxUpdateFeed.onChecked = function(aName, aURL) {
	free();
	hxUpdateFeed.downloadList = new Array();
	hxUpdateFeed.nameList = new Array();
	hxUpdateFeed.urlList = new Array();
	hxUpdateFeed.localPathList = new Array();
	hxUpdateFeed.localHTMLPathList = new Array();
	hxUpdateFeed.descriptionList = new Array();
	hxUpdateFeed.lastModifiedList = new Array();
	statusLabel.value = " ";
    }

    hxUpdateFeed.begin(prefFolder, false);
}


/**************************************************************************
* This function Unsets the LastVisitDate for the selected Feed
**************************************************************************/
function openOPMLWizard() {
    dump("openOPMLWizard(): Opening OPML Wizard.\n");

    // Open a new modal dialog showing the settingseditor xul as chrome
    // pass it a reference to the feedsTree
    var wizULR = "chrome://habarixenu/content/opml/opmlwizard.xul";
    window.openDialog(wizULR,"","chrome,modal, width=450, height=450", feedsTree);

    dump("openOPMLWizard(): Finished opening Settings editor\n");
    return true;
}


/**************************************************************************
* This function marks all feeds read
**************************************************************************/
function catchup() {
    var feedList = new Array();

    feedList = getBMDSCChildren(prefFolder);
    for (var i = 0; i < feedList.length; i++) {
	var aInput = (typeof(feedList[i]) == "string") ? RDF.GetResource(feedList[i]) : feedList[i];
	var aArcDescriptionURI = RDF.GetResource(NC_NS + "Description");
	var description = getBMDSProperty(aInput, aArcDescriptionURI);

	var curStatus = description.split(" -- ")[0];
	var curDesc = description.split(" -- ")[1];
	if (curStatus == "hxnew") {
	    var newDescription = "hxold -- " + curDesc;

	    try {
		setBMDSProperty(aInput, aArcDescriptionURI, newDescription);
	    } catch (e) {
		alert(e);
	    }
	}
    }
}
